package com.frist008.nixlibrary;

import android.net.Uri;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentActivity;
import android.support.v7.widget.RecyclerView;
import android.widget.AbsListView;
import android.widget.ExpandableListView;

import com.frist008.nixlibrary.database.Sort;
import com.frist008.nixlibrary.utils.NixException;
import com.frist008.nixlibrary.utils.NixWrapperUri;

import se.emilsjolander.stickylistheaders.StickyListHeadersListView;

/**
 * It simple builder for register {@link android.support.v4.content.CursorLoader}. For registration
 * of the loader use the static method {@link #create} and after then use a {@link #register}. You
 * can used other methods for configure his a query to database
 */
@SuppressWarnings("unused")
public class NixLoaderBuilder {

	@Nullable
	final Fragment fragment;
	@NonNull
	final FragmentActivity activity;
	@NonNull
	final Uri uri;
	@Nullable
	String[] projection;
	final Sort sort = new Sort();
	@Nullable
	String select;
	@Nullable
	String[] args;
	@Nullable
	OnChangeCursorBeforeListener listenerBefore;
	@Nullable
	OnChangeCursorAfterListener listenerAfter;
	@Nullable
	OnChangeWhereListener whereListener;

	NixLoaderBuilder(@NonNull FragmentActivity activity,
	                 @Nullable Fragment fragment,
	                 @NonNull Uri uri) {
		this.activity = activity;
		this.fragment = fragment;
		this.uri = uri;
	}

	/**
	 * @param nixUri   wrapper uri of table or view
	 * @param fragment with {@link FragmentActivity} for register of loader
	 * @return this builder
	 */
	public static NixLoaderBuilder create(@NonNull NixWrapperUri nixUri,
	                                      @NonNull Fragment fragment) {
		return create(nixUri.getUri(), fragment);
	}

	/**
	 * @param uri      of table or view
	 * @param fragment for register of loader
	 * @return this builder
	 */
	public static NixLoaderBuilder create(@NonNull Uri uri, @NonNull Fragment fragment) {
		return new NixLoaderBuilder(fragment.getActivity(), fragment, uri);
	}

	/**
	 * @param nixUri   wrapper uri of table or view
	 * @param activity for register of loader
	 * @return this builder
	 */
	public static NixLoaderBuilder create(@NonNull NixWrapperUri nixUri,
	                                      @NonNull FragmentActivity activity) {
		return create(nixUri.getUri(), activity);
	}

	/**
	 * @param uri      of table or view
	 * @param activity for register of loader
	 * @return this builder
	 */
	public static NixLoaderBuilder create(@NonNull Uri uri, @NonNull FragmentActivity activity) {
		return new NixLoaderBuilder(activity, null, uri);
	}

	/**
	 * @param projection A list of which columns to return. Passing null will return all columns,
	 *                   which is discouraged to prevent reading data from storage that isn't going
	 *                   to be used.
	 * @return this builder
	 */
	public NixLoaderBuilder projection(@Nullable String... projection) {
		this.projection = (projection == null || projection.length == 0) ? null : projection;
		return this;
	}

	/**
	 * How to order the rows, formatted as an SQL ORDER BY clause (excluding the ORDER BY itself).
	 * You can call it method or {@link #sortDesc} again after it calling of method
	 *
	 * @param field  A first column for sorting
	 * @param fields A list of which columns for sorting
	 * @return this builder
	 */
	public NixLoaderBuilder sortAsc(@NonNull String field, @Nullable String... fields) {
		this.sort.addAsc(field, fields);
		return this;
	}

	/**
	 * How to order the rows, formatted as an SQL ORDER BY clause (excluding the ORDER BY itself).
	 * You can call it method or {@link #sortAsc} again after it calling of method
	 *
	 * @param field  A first column for sorting
	 * @param fields A list of which columns for sorting
	 * @return this builder
	 */
	public NixLoaderBuilder sortDesc(@NonNull String field, @Nullable String... fields) {
		this.sort.addDesc(field, fields);
		return this;
	}

	/**
	 * Use this a method for query to database if 'WHERE' will not modifiable else use {@link
	 * #setWhereListener}
	 *
	 * @param select A filter declaring which rows to return, formatted as an SQL WHERE clause
	 *               (excluding the WHERE itself). Passing null will return all rows for the given
	 *               table.
	 * @param args   You may include '?'s in selection, which will be replaced by the values from
	 *               selectionArgs, in order that they appear in the selection. The values will be
	 *               bound as Strings.
	 * @return this builder
	 */
	public NixLoaderBuilder where(@Nullable String select, @Nullable String... args) {
		this.select = select;
		this.args = args != null && args.length == 0 ? null : args;
		return this;
	}

	/**
	 * Use this a method for query to database if 'WHERE' will not modifiable else use {@link
	 * #setWhereListener}
	 *
	 * @param select A filter declaring which rows to return, formatted as an SQL WHERE clause
	 *               (excluding the WHERE itself). Passing null will return all rows for the given
	 *               table.
	 * @return this builder
	 */
	public NixLoaderBuilder where(@Nullable String select) {
		this.select = select;
		return this;
	}

	/**
	 * Use this a method for query to database if 'WHERE' will modifiable else use {@link #where}
	 *
	 * @param whereListener It called every time the loader is initialized to create a new database
	 *                      query
	 * @return this builder
	 */
	public NixLoaderBuilder setWhereListener(@NonNull OnChangeWhereListener whereListener) {
		this.whereListener = whereListener;
		return this;
	}

	/**
	 * If you using list or a grid then method of the interface will called before calling {@link
	 * LoaderUtils#changeCursor}. If you not using list or a grid then method of a interface will
	 * called at the same time, and that {@link #changeCursorAfterListener}
	 *
	 * @param changeCursorListener see {@link android.support.v4.app.LoaderManager.LoaderCallbacks#onLoadFinished}
	 * @return this builder
	 */
	public NixLoaderBuilder changeCursorBeforeListener(
			@NonNull OnChangeCursorBeforeListener changeCursorListener) {
		listenerBefore = changeCursorListener;
		return this;
	}

	/**
	 * If you using list or a grid then method of the interface will called after calling {@link
	 * LoaderUtils#changeCursor}. If you not using list or a grid then method of a interface will
	 * called at the same time, and that {@link #changeCursorBeforeListener}
	 *
	 * @param changeCursorListener see {@link android.support.v4.app.LoaderManager.LoaderCallbacks#onLoadFinished}
	 * @return this builder
	 */
	public NixLoaderBuilder changeCursorAfterListener(
			@NonNull OnChangeCursorAfterListener changeCursorListener) {
		listenerAfter = changeCursorListener;
		return this;
	}

	/**
	 * Instantiate and return a new Loader for the given ID.
	 *
	 * @param listView For of adapter this view will called changeCursor
	 * @param loaderId The ID whose loader is to be created.
	 * @return Return a new Loader instance that is ready to start loading.
	 */
	@SuppressWarnings("unchecked")
	public NixCursorLoader<AbsListView> register(@NonNull AbsListView listView, int loaderId) {
		return new NixCursorListLoader(fragment,
		                               activity,
		                               listView,
		                               loaderId,
		                               listenerBefore,
		                               listenerAfter,
		                               whereListener,
		                               uri,
		                               projection,
		                               sort,
		                               select,
		                               args);
	}

	/**
	 * Instantiate and return a new Loader for the given ID.
	 *
	 * @param loaderId The ID whose loader is to be created.
	 * @return Return a new Loader instance that is ready to start loading.
	 */
	@SuppressWarnings("unchecked")
	public NixCursorLoader register(int loaderId) {
		if (listenerAfter == null && listenerBefore == null) {
			throw new NixException("To do this, the loader has not been assigned to any one of"
			                       + " the listeners. Please use changeCursorAfterListener"
			                       + "or changeCursorBeforeListener");
		}
		return new NixCursorLoader<>(fragment,
		                             activity,
		                             null,
		                             loaderId,
		                             listenerBefore,
		                             listenerAfter,
		                             whereListener,
		                             uri,
		                             projection,
		                             sort,
		                             select,
		                             args);
	}

	/**
	 * Instantiate and return a new Loader for the given ID.
	 *
	 * @param expandableListView For of adapter this view will called changeCursor
	 * @param loaderId           The ID whose loader is to be created.
	 * @return Return a new Loader instance that is ready to start loading.
	 */
	@SuppressWarnings("unchecked")
	public NixCursorLoader<ExpandableListView> register(@NonNull
	                                                    ExpandableListView expandableListView,
	                                                    int loaderId) {
		return new NixCursorExpandableListLoader(fragment,
		                                         activity,
		                                         expandableListView,
		                                         loaderId,
		                                         listenerBefore,
		                                         listenerAfter,
		                                         whereListener,
		                                         uri,
		                                         projection,
		                                         sort,
		                                         select,
		                                         args);
	}

	/**
	 * Instantiate and return a new Loader for the given ID.
	 *
	 * @param recyclerView For of adapter this view will called changeCursor
	 * @param loaderId     The ID whose loader is to be created.
	 * @return Return a new Loader instance that is ready to start loading.
	 */
	@SuppressWarnings("unchecked")
	public NixCursorRecyclerLoader registerRecycler(@NonNull RecyclerView recyclerView,
	                                                int loaderId) {
		return new NixCursorRecyclerLoader(fragment,
		                                   activity,
		                                   recyclerView,
		                                   loaderId,
		                                   listenerBefore,
		                                   listenerAfter,
		                                   whereListener,
		                                   uri,
		                                   projection,
		                                   sort,
		                                   select,
		                                   args);
	}

	/**
	 * Instantiate and return a new Loader for the given ID.
	 *
	 * @param listView For of adapter this view will called changeCursor
	 * @param loaderId The ID whose loader is to be created.
	 * @return Return a new Loader instance that is ready to start loading.
	 */
	@SuppressWarnings("unchecked")
	public NixCursorStickyListHeadersLoader registerStickyListHeaders(
			@NonNull StickyListHeadersListView listView,
			int loaderId) {
		return new NixCursorStickyListHeadersLoader(fragment,
		                                            activity,
		                                            listView,
		                                            loaderId,
		                                            listenerBefore,
		                                            listenerAfter,
		                                            whereListener,
		                                            uri,
		                                            projection,
		                                            sort,
		                                            select,
		                                            args);
	}
}
